module GoodServices
  module Generators
    class ServiceGenerator < Rails::Generators::NamedBase
      desc "Creates a GoodServices basic service"

      def create_service
        if constants.size == 1
          create_file service_path, namespaceless_template
        elsif constants.size == 2
          create_file service_path, namespaced_template
        else
          puts
          puts "[Error] This generator only supports one level of namespacing!"
        end
      end

      private

      def namespaceless_template
        get_template("namespaceless_service").result(binding)
      end

      def namespaced_template
        get_template("namespaced_service").result(binding)
      end

      def get_template(file)
        path = File.expand_path("../templates/#{file}.erb", __FILE__)
        ERB.new(File.read(path))
      end

      def service_path
        if service_module
          "app/services/#{module_name}/#{service_name}.rb"
        else
          "app/services/#{service_name}.rb"
        end
      end

      #
      # Argument Helpers
      #
      def constants
        class_name.split("::")
      end

      def service_module
        @_service_module ||= constants.first if constants.size > 1
      end

      def service_class
        @_service_class ||= class_name.split("::").last
      end

      def service_name
        @_service_name ||= service_class.underscore
      end

      def module_name
        @_module_name ||= service_module.underscore
      end
    end
  end
end
