module GoodServices
  class Base
    attr_reader :record, :collection, :error

    #
    # Inheritance wizardry
    #
    class << self
      attr_accessor :rescuable_list, :default_rescues

      DEFAULT_RESCUES = []

      def inherit_variables(rescuable_list, default_rescues)
        @rescuable_list  = rescuable_list  || []
        @default_rescues = default_rescues || DEFAULT_RESCUES
      end

      def inherited(subclass)
        subclass.inherit_variables(@rescuable_list, @default_rescues)
      end
    end

    #
    # Helpers for instancing and running the Service
    #
    def self.run(*args)
      self.new(*args).run
    end

    def self.run!(*args)
      self.new(*args).run!
    end

    #
    # Runs the validations and perform methods
    # Returns true or raise an exception
    #
    def run!
      validate
      perform
      return true
    end

    #
    # Runs the validations and perform methods rescuing from known exceptions
    # Returns either true or false
    #
    def run
      validate
      perform
      return true
    rescue *self.class.acceptable_exceptions => @error
      return false
    end

    def validate
    end

    def perform
    end

    #
    # This configuration method adds custom exceptions to the rescuable list
    #
    def self.rescuable_from(*rescuable_exceptions)
      @rescuable_list = rescuable_exceptions
    end

    #
    # This method returns either the set rescuable_from or the defaults
    #
    def self.acceptable_exceptions
      (@rescuable_list + @default_rescues).uniq
    end
  end
end
